import torch
import torch.nn as nn
import torch.nn.functional as F


class Siamese(nn.Module):

    def __init__(self):
        super(Siamese, self).__init__()
        self.conv = nn.Sequential(
            nn.Conv2d(1, 64, 10),  # 64@96*96
            nn.ReLU(inplace=True),
            nn.MaxPool2d(2),  # 64@48*48
            nn.Conv2d(64, 128, 7),
            nn.ReLU(),    # 128@42*42
            nn.MaxPool2d(2),   # 128@21*21
            nn.Conv2d(128, 128, 4),
            nn.ReLU(), # 128@18*18
            nn.MaxPool2d(2), # 128@9*9
            nn.Conv2d(128, 256, 4),
            nn.ReLU(),   # 256@6*6
        )
        self.liner = nn.Sequential(nn.Linear(9216, 4096), nn.Sigmoid())
        self.out = nn.Linear(4096, 1)

    def forward_one(self, x):
        x = self.conv(x)
        x = x.view(x.size()[0], -1)
        x = self.liner(x)
        return x

    def forward(self, x1, x2):
        out1 = self.forward_one(x1)
        out2 = self.forward_one(x2)
        dis = torch.abs(out1 - out2)
        out = self.out(dis)
        return torch.sigmoid(out)


# for test
if __name__ == '__main__':
    net = Siamese()
    print(net)
    print(list(net.parameters()))




from pathlib import Path

from tqdm import tqdm
import requests

def download_model(model_path: Path, version="v1.0.0", block_size=1024):
    model_url = f"https://github.com/joeyism/siamese-pytorch/releases/download/{version}/siamese-model.pt"
    response = requests.get(model_url, stream=True)
    total_size_in_bytes= int(response.headers.get('content-length', 0))
    progress_bar = tqdm(desc="Downloading model", total=total_size_in_bytes, unit='iB', unit_scale=True)
    with open(model_path.as_posix(), 'wb') as file:
        for data in response.iter_content(block_size):
            progress_bar.update(len(data))
            file.write(data)
    progress_bar.close()
    if total_size_in_bytes != 0 and progress_bar.n != total_size_in_bytes:
        print("ERROR, something went wrong")



from typing import Union
from pathlib import Path

import numpy as np
import torch
from PIL import Image
from torchvision import transforms

from .model import Siamese
from .download import download_model

ImageType = Union[Image.Image, np.ndarray]
DEFAULT_IMAGE_SIZE = (105, 105)
MODEL_FOLDER = Path(__file__).parents[1] / "models"
MODEL_FOLDER.mkdir(parents=True, exist_ok=True)
MODEL_PATH = MODEL_FOLDER / "siamese-model.pt"
if not MODEL_PATH.exists():
    download_model(MODEL_PATH)

model = Siamese()
model_dict = torch.load(MODEL_PATH, map_location="cpu")
model.load_state_dict(model_dict)

transformer = transforms.Compose(
    [
        transforms.Grayscale(),
        transforms.ToTensor(),
    ]
)

def _pil_image_(image: ImageType) -> Image:
    return Image.fromarray(np.array(image)).resize(DEFAULT_IMAGE_SIZE)

def is_similar(image1: ImageType, image2: ImageType, threshold=0.5):
    pil_image1 = _pil_image_(image1)
    pil_image2 = _pil_image_(image2)

    image_tensor1 = transformer(pil_image1).unsqueeze(0)
    image_tensor2 = transformer(pil_image2).unsqueeze(0)

    results = model(image_tensor1, image_tensor2)
    return results.detach().cpu().numpy()[0][0] > threshold



from pathlib import Path
import argparse

import cv2

from .compare import is_similar


def cli():
    parser = argparse.ArgumentParser(description='Compares two images')
    parser.add_argument('image1', metavar="Image1-Path", type=Path, help='first image in comparison')
    parser.add_argument('image2', metavar="Image2-Path", type=Path, help='second image in comparison')
    parser.add_argument('--threshold', type=float, default=0.5, help='threshold limit (default: 0.5)')
    args = parser.parse_args()
    main(args.image1, args.image2)

def main(image1_path: Path, image2_path: Path):
    image1 = cv2.imread(image1_path.as_posix())
    image2 = cv2.imread(image2_path.as_posix())
    if is_similar(image1, image2):
        print(f"{image1_path.name} and {image2_path.name} are similar")
    else:
        print(f"{image1_path.name} and {image2_path.name} are not similar")

if __name__ == "__main__":
    cli()



from setuptools import find_packages, setup
import re

PACKAGE_NAME = "image-comparer"
PACKAGE_NAME_UNDERLINE = PACKAGE_NAME.replace("-", "_")

version = re.search(
    '^__version__\s*=\s*"(.*)"',
    open(f'{PACKAGE_NAME_UNDERLINE}/__init__.py').read(),
    re.M
    ).group(1)

setup(
    name=PACKAGE_NAME,
    packages=find_packages(exclude=["tests"]),
    version=version,
    description='Compares two images using siamese networks',
    long_description=open("README.md", "r").read(),
    long_description_content_type='text/markdown',
    author='joeyism',
    url='https://github.com/joeyism/py-image-comparer',
    download_url='https://github.com/joeyism/py-image-comparer/archive/{}.tar.gz'.format(version),
    install_requires=[package for package in open("requirements.txt").read().split("\n")],
    keywords=["pytorch", "torch", "machine", "learning", "image", "compare", "comparer", "siamese", "network", "networks"],
    entry_points={
        "console_scripts": [
            f"image-compare = {PACKAGE_NAME_UNDERLINE}.cli:cli",
        ]
    },
    package_data={"": ["*.txt", "*.cfg"]},
    include_package_data=True,
)
